-- Switch to the SingleDB_Broker database
USE SingleDB_Broker;
GO

-- Begin a conversation and send a request message
DECLARE @InitDlgHandle UNIQUEIDENTIFIER;
DECLARE @RequestMsg NVARCHAR(100);

BEGIN TRANSACTION;

BEGIN DIALOG @InitDlgHandle
     FROM SERVICE
      [//SingleDB_Broker/InitiatorService]
     TO SERVICE
      N'//SingleDB_Broker/TargetService'
     ON CONTRACT
      [//SingleDB_Broker/SampleContract]
     WITH
         ENCRYPTION = OFF;

SELECT @RequestMsg =
       N'<RequestMsg>Message for Target service.</RequestMsg>';

SEND ON CONVERSATION @InitDlgHandle
     MESSAGE TYPE 
     [//SingleDB_Broker/RequestMessage]
     (@RequestMsg);

SELECT @RequestMsg AS SentRequestMsg;

COMMIT TRANSACTION;
GO

-- View the conversation we created
SELECT *
FROM sys.conversation_groups;
GO

-- View the message in the targets queue
SELECT *, 
	CAST(message_body AS XML) AS message_body_xml
FROM SingleDB_Broker_TargetQueue;
GO

-- Receive the request and send a reply
DECLARE @RecvReqDlgHandle UNIQUEIDENTIFIER;
DECLARE @RecvReqMsg NVARCHAR(100);
DECLARE @RecvReqMsgName sysname;

BEGIN TRANSACTION;

WAITFOR
( RECEIVE TOP(1)
    @RecvReqDlgHandle = conversation_handle,
    @RecvReqMsg = message_body,
    @RecvReqMsgName = message_type_name
  FROM SingleDB_Broker_TargetQueue
), TIMEOUT 1000;

SELECT @RecvReqMsg AS ReceivedRequestMsg;

IF @RecvReqMsgName =
   N'//SingleDB_Broker/RequestMessage'
BEGIN
     DECLARE @ReplyMsg NVARCHAR(100);
     SELECT @ReplyMsg =
     N'<ReplyMsg>Message for Initiator service.</ReplyMsg>';
 
     SEND ON CONVERSATION @RecvReqDlgHandle
          MESSAGE TYPE 
          [//SingleDB_Broker/ReplyMessage]
          (@ReplyMsg);
END

SELECT @ReplyMsg AS SentReplyMsg;

COMMIT TRANSACTION;
GO

-- Check for the message in the targets queue
SELECT *, 
	CAST(message_body AS XML) AS message_body_xml
FROM SingleDB_Broker_TargetQueue;
GO

-- Check for the reply message in the initiator queue
SELECT *, 
	CAST(message_body AS XML) AS message_body_xml
FROM SingleDB_Broker_InitiatorQueue;
GO



-- Receive the reply and end the conversation
DECLARE @RecvReplyMsg NVARCHAR(100);
DECLARE @RecvReplyDlgHandle UNIQUEIDENTIFIER;
DECLARE @RecvReplyMsgName sysname;

BEGIN TRANSACTION;

WAITFOR
( RECEIVE TOP(1)
    @RecvReplyDlgHandle = conversation_handle,
    @RecvReplyMsg = message_body,
	@RecvReplyMsgName = message_type_name
  FROM SingleDB_Broker_InitiatorQueue
), TIMEOUT 1000;

IF @RecvReplyMsgName =
   N'//SingleDB_Broker/ReplyMessage'
BEGIN
	END CONVERSATION @RecvReplyDlgHandle;
END

SELECT @RecvReplyMsg AS ReceivedReplyMsg;

COMMIT TRANSACTION;
GO


-- Check for a message in the targets queue
SELECT *, 
	CAST(message_body AS XML) AS message_body_xml
FROM SingleDB_Broker_TargetQueue;
GO

-- Check for a  message in the initiator queue
SELECT *, 
	CAST(message_body AS XML) AS message_body_xml
FROM SingleDB_Broker_InitiatorQueue;
GO



-- Receive the End Dialog and clean up
DECLARE @RecvEndDialogDlgHandle UNIQUEIDENTIFIER;
DECLARE @RecvEndDialogMsg NVARCHAR(100);
DECLARE @RecvEndDialogMsgName sysname;

BEGIN TRANSACTION;

WAITFOR
( RECEIVE TOP(1)
    @RecvEndDialogDlgHandle = conversation_handle,
    @RecvEndDialogMsg = message_body,
    @RecvEndDialogMsgName = message_type_name
  FROM SingleDB_Broker_TargetQueue
), TIMEOUT 1000;


IF @RecvEndDialogMsgName =
   N'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
BEGIN
     END CONVERSATION @RecvEndDialogDlgHandle;
END

COMMIT TRANSACTION;
GO

-- Check for a message in the targets queue
SELECT *, 
	CAST(message_body AS XML) AS message_body_xml
FROM SingleDB_Broker_TargetQueue;
GO

-- Check for a  message in the initiator queue
SELECT *, 
	CAST(message_body AS XML) AS message_body_xml
FROM SingleDB_Broker_InitiatorQueue;
GO

-- Check the conversation we created
SELECT *
FROM sys.conversation_groups;
GO