-- Create the databases and and do not set the trustworthy option
USE master;
GO

IF DB_ID('TargetDB') IS NOT NULL
BEGIN
	ALTER DATABASE TargetDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE TargetDB;
END
GO
CREATE DATABASE TargetDB;
GO

ALTER DATABASE TargetDB
      SET NEW_BROKER;
GO

IF DB_ID('InitiatorDB') IS NOT NULL
BEGIN
	ALTER DATABASE InitiatorDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE InitiatorDB;
END
GO
CREATE DATABASE InitiatorDB;
GO

ALTER DATABASE InitiatorDB
      SET NEW_BROKER;
GO

-- Switch to the TargetDB database and create broker objects
USE TargetDB;
GO

CREATE MASTER KEY ENCRYPTION BY PASSWORD = N'$tr0ngp@SSw0rd';
GO
 
-- Create TargetDB User without login
CREATE USER TargetDBUser WITHOUT LOGIN
 
-- Create a Cert for the Initiator user
CREATE CERTIFICATE TargetDBCert
AUTHORIZATION TargetDBUser
WITH SUBJECT = N'TargetDB Certificate', 
	 EXPIRY_DATE = N'12/31/2020';
 
!! DEL "C:\SQLData\Backups\TargetDBCert.cer"

-- Backup the cert up so the TargetDB database can import the cert (public key)
BACKUP CERTIFICATE TargetDBCert 
TO FILE = N'C:\SQLData\Backups\TargetDBCert.cer';
GO

-- Create the message types
CREATE MESSAGE TYPE [//MultiDB_Broker/RequestMessage]
       VALIDATION = WELL_FORMED_XML;
GO

CREATE MESSAGE TYPE [//MultiDB_Broker/ReplyMessage]
       VALIDATION = WELL_FORMED_XML;
GO

-- Create the contract
CREATE CONTRACT [//MultiDB_Broker/SimpleContract]
      ([//MultiDB_Broker/RequestMessage] SENT BY INITIATOR,
       [//MultiDB_Broker/ReplyMessage] SENT BY TARGET );
GO

-- Create the target queue and service
CREATE QUEUE MultiDB_Broker_TargetQueue;

CREATE SERVICE [//MultiDB_Broker/TargetService]
AUTHORIZATION TargetDBUser
ON QUEUE MultiDB_Broker_TargetQueue ([//MultiDB_Broker/SimpleContract]);
GO

USE msdb
GO

IF EXISTS (SELECT 1
		   FROM sys.routes
		   WHERE name = N'InitiatorRoute')
	DROP ROUTE InitiatorRoute;
GO

CREATE ROUTE InitiatorRoute 
WITH SERVICE_NAME = N'//MultiDB_Broker/InitiatorService', 
ADDRESS = N'LOCAL';
GO


--Switch to the InitiatorDB database and create broker objects
USE InitiatorDB;
GO

CREATE MASTER KEY ENCRYPTION BY PASSWORD = N'$tr0ngp@SSw0rd';
GO
 
-- Create InitiatorDB User without login
CREATE USER InitiatorDBUser WITHOUT LOGIN;
GO
 
-- Create a Cert for the Initiator user
CREATE CERTIFICATE InitiatorDBCert
AUTHORIZATION InitiatorDBUser
WITH SUBJECT = N'InitiatorDB Certificate', 
	 EXPIRY_DATE = N'12/31/2020';
GO

!! DEL "C:\SQLData\Backups\InitiatorDBCert.cer"
 
-- Backup the cert up so the InitiatorDB database can import the cert (public key)
BACKUP CERTIFICATE InitiatorDBCert 
TO FILE = N'C:\SQLData\Backups\InitiatorDBCert.cer';
GO

-- Create the message types
CREATE MESSAGE TYPE [//MultiDB_Broker/RequestMessage]
       VALIDATION = WELL_FORMED_XML;
GO

CREATE MESSAGE TYPE [//MultiDB_Broker/ReplyMessage]
       VALIDATION = WELL_FORMED_XML;
GO

-- Create the contract
CREATE CONTRACT [//MultiDB_Broker/SimpleContract]
	([//MultiDB_Broker/RequestMessage] SENT BY INITIATOR,
	 [//MultiDB_Broker/ReplyMessage] SENT BY TARGET);
GO

-- Create the initiator queue and service
CREATE QUEUE MultiDB_Broker_InitiatorQueue;

CREATE SERVICE [//MultiDB_Broker/InitiatorService]
AUTHORIZATION InitiatorDBUser
       ON QUEUE MultiDB_Broker_InitiatorQueue;
GO

USE msdb
GO

IF EXISTS (SELECT 1
		   FROM sys.routes
		   WHERE name = N'TargetDBRoute')
	DROP ROUTE TargetDBRoute;
GO

CREATE ROUTE TargetDBRoute 
WITH SERVICE_NAME = N'//MultiDB_Broker/TargetService', 
ADDRESS = N'LOCAL';
GO

USE InitiatorDB;
GO

CREATE USER TargetDBUser WITHOUT LOGIN;
 
-- Create a Cert from the external cert file
CREATE CERTIFICATE TargetDBCert
AUTHORIZATION TargetDBUser
FROM FILE = N'C:\SQLData\Backups\TargetDBCert.cer';
GO
 
-- Create a remote service binding (only needs to be done on the initator)
CREATE REMOTE SERVICE BINDING TargetDBBinding
TO SERVICE N'//MultiDB_Broker/TargetService'
WITH USER = TargetDBUser;
GO

GRANT SEND ON SERVICE::[//MultiDB_Broker/InitiatorService] TO TargetDBUser;
GO

-- Set up the TargetDB user using the TargetDB exported certificate
USE TargetDB;
GO

-- Create Initiator User without Login
CREATE USER InitiatorDBUser WITHOUT LOGIN;
 
-- Create a Cert from the external cert file
CREATE CERTIFICATE InitiatorDBCert
AUTHORIZATION InitiatorDBUser
FROM FILE = N'C:\SQLData\Backups\InitiatorDBCert.cer';
GO
 
GRANT SEND ON SERVICE::[//MultiDB_Broker/TargetService] TO InitiatorDBUser;
GO
