-- NOTE: THIS SCRIPT MUST BE RUN IN SQLCMD MODE TO FUNCTION PROPERLY
:CONNECT Jessie

--Create a Service Broker endpoint
USE master;
GO

IF EXISTS (SELECT * 
		   FROM master.sys.endpoints
           WHERE name = N'SSB_Endpoint')
    DROP ENDPOINT SSB_Endpoint;
GO

CREATE ENDPOINT SSB_Endpoint
STATE = STARTED
AS TCP ( LISTENER_PORT = 4022 )
FOR SERVICE_BROKER (AUTHENTICATION = WINDOWS );
GO

-- Grant connect to the service account
GRANT CONNECT ON ENDPOINT::SSB_EndPoint to [SQLskillsDemos\SQLServiceAcct] ;
GO

-- Create the Target database
USE master;
GO
IF DB_ID(N'TargetDB') IS NOT NULL
BEGIN
	ALTER DATABASE TargetDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE TargetDB;
END
GO
CREATE DATABASE TargetDB;
GO
USE TargetDB;
GO

-- Create the master key
CREATE MASTER KEY ENCRYPTION BY PASSWORD = N'$tr0ngp@$$w0rd';
GO

-- Create the Target user
CREATE USER TargetUser WITHOUT LOGIN;
GO

CREATE CERTIFICATE TargetCertificate
AUTHORIZATION TargetUser
    FROM FILE = N'\\Woody\Backups\TargetCertificate.cer'
    WITH PRIVATE KEY (FILE =  N'\\Woody\Backups\TargetCertificate.key', 
    DECRYPTION BY PASSWORD = '997jkhUbhk$w4ez0876hKHJH5gh');
GO 

-- Create the message types
CREATE MESSAGE TYPE [//MultipleInstances/RequestMessage]
VALIDATION = WELL_FORMED_XML;

CREATE MESSAGE TYPE [//MultipleInstances/ReplyMessage]
VALIDATION = WELL_FORMED_XML;
GO

-- Create the contract
CREATE CONTRACT [//MultipleInstances/SimpleContract]
	([//MultipleInstances/RequestMessage] SENT BY INITIATOR,
     [//MultipleInstances/ReplyMessage] SENT BY Target);
GO

-- Create the Target queue and service
CREATE QUEUE TargetQueue;

CREATE SERVICE [//MultipleInstances/TargetService]
AUTHORIZATION TargetUser
ON QUEUE TargetQueue ([//MultipleInstances/SimpleContract]);
GO

