-- Create the databases and set the trustworthy option
USE master;
GO

IF DB_ID('TargetDB') IS NOT NULL
BEGIN
	ALTER DATABASE TargetDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE TargetDB;
END
GO
CREATE DATABASE TargetDB;
GO
ALTER DATABASE TargetDB SET TRUSTWORTHY ON;
GO

ALTER DATABASE TargetDB
      SET ENABLE_BROKER;
GO

IF DB_ID('InitiatorDB') IS NOT NULL
BEGIN
	ALTER DATABASE InitiatorDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE InitiatorDB;
END
GO
CREATE DATABASE InitiatorDB;
GO
ALTER DATABASE InitiatorDB SET TRUSTWORTHY ON;
GO

ALTER DATABASE InitiatorDB
      SET ENABLE_BROKER;
GO

-- Switch to the TargetDB database and create broker objects
USE TargetDB;
GO

-- Create the message types
CREATE MESSAGE TYPE [//MultiDB_Broker/RequestMessage]
       VALIDATION = WELL_FORMED_XML;
CREATE MESSAGE TYPE [//MultiDB_Broker/ReplyMessage]
       VALIDATION = WELL_FORMED_XML;
GO

-- Create the contract
CREATE CONTRACT [//MultiDB_Broker/SimpleContract]
      ([//MultiDB_Broker/RequestMessage]
         SENT BY INITIATOR,
       [//MultiDB_Broker/ReplyMessage]
         SENT BY TARGET
      );
GO

-- Create the target queue and service
CREATE QUEUE MultiDB_Broker_TargetQueue;

CREATE SERVICE [//MultiDB_Broker/TargetService]
       ON QUEUE MultiDB_Broker_TargetQueue
       ([//MultiDB_Broker/SimpleContract]);
GO


--Switch to the InitiatorDB database and create broker objects
USE InitiatorDB;
GO

-- Create the message types
CREATE MESSAGE TYPE [//MultiDB_Broker/RequestMessage]
       VALIDATION = WELL_FORMED_XML;
CREATE MESSAGE TYPE [//MultiDB_Broker/ReplyMessage]
       VALIDATION = WELL_FORMED_XML;
GO

-- Create the contract
CREATE CONTRACT [//MultiDB_Broker/SimpleContract]
      ([//MultiDB_Broker/RequestMessage]
         SENT BY INITIATOR,
       [//MultiDB_Broker/ReplyMessage]
         SENT BY TARGET
      );
GO

-- Create the initiator queue and service
CREATE QUEUE MultiDB_Broker_InitiatorQueue;

CREATE SERVICE [//MultiDB_Broker/InitiatorService]
       ON QUEUE MultiDB_Broker_InitiatorQueue;
GO