-- Switch to the TargetDB database
USE TargetDB;
GO

CREATE PROCEDURE TargetQueue_ActivationProcedure
AS

  DECLARE @conversation_handle UNIQUEIDENTIFIER;
  DECLARE @message_body XML;
  DECLARE @message_type_name sysname;

  WHILE (1=1)
  BEGIN

    BEGIN TRANSACTION;

    WAITFOR
    ( RECEIVE TOP(1)
        @conversation_handle = conversation_handle,
        @message_body = CAST(message_body AS XML),
        @message_type_name = message_type_name
      FROM MultiDB_Broker_TargetQueue
    ), TIMEOUT 5000;

    IF (@@ROWCOUNT = 0)
    BEGIN
      ROLLBACK TRANSACTION;
      BREAK;
    END

    IF @message_type_name = N'//MultiDB_Broker/RequestMessage'
    BEGIN
		INSERT INTO MessagesProcessed(XMLValue)
		SELECT @message_body.value('(RequestMsg[1])', 'int')

		WAITFOR DELAY '00:00:00.050';

       DECLARE @reply_message_body XML;
	   SET @reply_message_body = N'<ReplyMsg>Processed value was '+CAST(@message_body.value('(RequestMsg[1])', 'int') AS NVARCHAR)+'</ReplyMsg>';
 
       SEND ON CONVERSATION @conversation_handle
              MESSAGE TYPE [//MultiDB_Broker/ReplyMessage] (@reply_message_body);
    END

	-- If end dialog message, end the dialog
    ELSE IF @message_type_name = N'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
    BEGIN
       END CONVERSATION @conversation_handle;
    END

	-- If error message, log and end conversation
    ELSE IF @message_type_name = N'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
    BEGIN
       END CONVERSATION @conversation_handle;
    END
      
    COMMIT TRANSACTION;

  END
GO

-- Alter the target queue to specify internal activation
ALTER QUEUE MultiDB_Broker_TargetQueue
    WITH ACTIVATION
    ( STATUS = ON,
      PROCEDURE_NAME = TargetQueue_ActivationProcedure,
      MAX_QUEUE_READERS = 10,
      EXECUTE AS SELF
    );
GO

SELECT * 
FROM MultiDB_Broker_TargetQueue WITH(NOLOCK);

SELECT *
FROM MessagesProcessed WITH(NOLOCK)
WHERE RowID <> XMLValue
ORDER BY RowID



