-- NOTE: THIS SCRIPT MUST BE RUN IN SQLCMD MODE TO FUNCTION PROPERLY
:CONNECT Buzz

-- Switch to the InitiatorDB database
USE InitiatorDB;
GO

CREATE PROCEDURE InitiatorQueue_ActivationProcedure
AS

  DECLARE @conversation_handle UNIQUEIDENTIFIER;
  DECLARE @message_body NVARCHAR(100);
  DECLARE @message_type_name sysname;

  WHILE (1=1)
  BEGIN

    BEGIN TRANSACTION;

    WAITFOR
    ( RECEIVE TOP(1)
        @conversation_handle = conversation_handle,
        @message_body = message_body,
        @message_type_name = message_type_name
      FROM InitiatorQueue
    ), TIMEOUT 5000;

    IF (@@ROWCOUNT = 0)
    BEGIN
      ROLLBACK TRANSACTION;
      BREAK;
    END

    IF @message_type_name = N'//MultipleInstances/ReplyMessage'
    BEGIN
		-- If necessary handle the reply here
		RAISERROR('Reply received from target', 10, 1) WITH LOG;
    END

	-- If end dialog message, end the dialog
    ELSE IF @message_type_name = N'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
    BEGIN
       END CONVERSATION @conversation_handle;
    END

	-- If error message, log and end conversation
    ELSE IF @message_type_name = N'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
    BEGIN
       END CONVERSATION @conversation_handle;
    END
      
    COMMIT TRANSACTION;

  END
GO

-- Alter the target queue to specify internal activation
ALTER QUEUE InitiatorQueue
    WITH ACTIVATION
    ( STATUS = ON,
      PROCEDURE_NAME = InitiatorQueue_ActivationProcedure,
      MAX_QUEUE_READERS = 10,
      EXECUTE AS SELF
    );
GO