-- NOTE: THIS SCRIPT MUST BE RUN IN SQLCMD MODE TO FUNCTION PROPERLY
:CONNECT Woody

-- Switch to the TargetDB database
USE TargetDB;
GO

ALTER PROCEDURE TargetQueue_ActivationProcedure
AS

  DECLARE 
     @conversation_handle UNIQUEIDENTIFIER,
     @message_sequence_number BIGINT,
     @message_type_name NVARCHAR(256),
     @message_body XML ;

  DECLARE @resultsTable TABLE(
     conversation_handle UNIQUEIDENTIFIER,
     message_sequence_number BIGINT,
     message_type_name NVARCHAR(256),
     message_body XML) ;

  WHILE (1=1)
  BEGIN

    BEGIN TRANSACTION;

    WAITFOR
    ( RECEIVE TOP(1000)
        conversation_handle,
		message_sequence_number,
		message_type_name,
        CAST(message_body  AS XML)
      FROM TargetQueue
	  INTO @resultsTable
    ), TIMEOUT 5000;

    IF (@@ROWCOUNT = 0)
    BEGIN
      ROLLBACK TRANSACTION;
      BREAK;
    END

	SELECT TOP(1) @message_type_name = message_type_name,
		@conversation_handle = conversation_handle
	FROM @ResultsTable;

    IF EXISTS (SELECT 1 FROM @ResultsTable WHERE message_type_name = N'EndOfStream')
    BEGIN
       END CONVERSATION @conversation_handle;
    END

	-- If end dialog message, end the dialog
    ELSE IF @message_type_name = N'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
    BEGIN
       END CONVERSATION @conversation_handle;
    END

	-- If error message, log and end conversation
    ELSE IF @message_type_name = N'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
    BEGIN
       END CONVERSATION @conversation_handle;
    END
      
    COMMIT TRANSACTION;

  END
GO

-- Alter the target queue to specify internal activation
ALTER QUEUE TargetQueue
    WITH ACTIVATION
    ( STATUS = ON,
      PROCEDURE_NAME = TargetQueue_ActivationProcedure,
      MAX_QUEUE_READERS = 10,
      EXECUTE AS SELF
    );
GO