-- Switch to the InitiatorDB database
USE InitiatorDB;
GO

-- Start a conversation and send a request message
DECLARE @conversation_handle UNIQUEIDENTIFIER;
DECLARE @message_body NVARCHAR(100);

BEGIN TRANSACTION;

BEGIN DIALOG @conversation_handle
     FROM SERVICE [//MultiDB_Broker/InitiatorService]
     TO SERVICE N'//MultiDB_Broker/TargetService'
     ON CONTRACT [//MultiDB_Broker/SimpleContract]
     WITH
         ENCRYPTION = OFF;

SELECT @message_body =
   N'<RequestMsg>Message for Target service.</RequestMsg>';

SEND ON CONVERSATION @conversation_handle
     MESSAGE TYPE [//MultiDB_Broker/RequestMessage]
      (@message_body);

SELECT @message_body AS SentRequestMsg;

COMMIT TRANSACTION;
GO

-- Switch to the TargetDB database
USE TargetDB;
GO

-- Receive the request and send a reply
DECLARE @conversation_handle UNIQUEIDENTIFIER;
DECLARE @message_body NVARCHAR(100);
DECLARE @message_type_name sysname;

BEGIN TRANSACTION;

WAITFOR
( RECEIVE TOP(1)
    @conversation_handle = conversation_handle,
    @message_body = message_body,
    @message_type_name = message_type_name
  FROM MultiDB_Broker_TargetQueue
), TIMEOUT 1000;

SELECT @message_body AS ReceivedRequestMsg;

IF @message_type_name =
   N'//MultiDB_Broker/RequestMessage'
BEGIN
     DECLARE @reply_message_body NVARCHAR(100);
     SELECT @reply_message_body =
        N'<ReplyMsg>Message for Initiator service.</ReplyMsg>';
 
     SEND ON CONVERSATION @conversation_handle
          MESSAGE TYPE
            [//MultiDB_Broker/ReplyMessage] (@reply_message_body);

     END CONVERSATION @conversation_handle;
END

SELECT @reply_message_body AS SentReplyMsg;

COMMIT TRANSACTION;
GO

-- Switch to the InitiatorDB database
USE InitiatorDB;
GO

-- Receive the reply and end the conversation
DECLARE @message_body NVARCHAR(100);
DECLARE @conversation_handle UNIQUEIDENTIFIER;

BEGIN TRANSACTION;

WAITFOR
( RECEIVE TOP(1)
    @conversation_handle = conversation_handle,
    @message_body = message_body
  FROM MultiDB_Broker_InitiatorQueue
), TIMEOUT 1000;

END CONVERSATION @conversation_handle;

-- Display recieved request.
SELECT @message_body AS ReceivedReplyMsg;

COMMIT TRANSACTION;
GO
