-- Switch to the InitiatorDB database
USE InitiatorDB;
GO

SELECT * FROM sys.transmission_queue;


-- Start a conversation and resend the broken request message
DECLARE @old_conversation_handle UNIQUEIDENTIFIER;
DECLARE @new_conversation_handle UNIQUEIDENTIFIER;
DECLARE @message_body XML;

-- Get the conversation handle for the message stuck in transmission
SELECT TOP 1 @old_conversation_handle = conversation_handle
FROM sys.transmission_queue
WHERE to_service_name = '//MultiDB_Broker/TargetService'
  AND from_service_name = N'//MultiDB_Broker/InitiatorService'

WHILE @old_conversation_handle IS NOT NULL
BEGIN

BEGIN TRANSACTION;

-- Create the new conversation
BEGIN DIALOG @new_conversation_handle
     FROM SERVICE [//MultiDB_Broker/InitiatorService]
     TO SERVICE N'//MultiDB_Broker/TargetService'
     ON CONTRACT [//MultiDB_Broker/SimpleContract]
     WITH
         ENCRYPTION = OFF;


	-- Need inner loop to handle multi-message conversations
	DECLARE @message_sequence_number INT;

	SELECT @message_sequence_number = MIN(message_sequence_number)
	FROM sys.transmission_queue
	WHERE to_service_name = '//MultiDB_Broker/TargetService'
	  AND from_service_name = N'//MultiDB_Broker/InitiatorService'
	  AND conversation_handle = @old_conversation_handle;

	SELECT @message_body = CAST(message_body AS XML)
	FROM sys.transmission_queue
	WHERE to_service_name = '//MultiDB_Broker/TargetService'
	  AND from_service_name = N'//MultiDB_Broker/InitiatorService'
	  AND conversation_handle = @old_conversation_handle
	  AND message_sequence_number = @message_sequence_number;

	WHILE @message_body IS NOT NULL
	BEGIN
		-- Send the mesage on the new conversation
		SEND ON CONVERSATION @new_conversation_handle
			 MESSAGE TYPE [//MultiDB_Broker/RequestMessage]
			  (@message_body);
	
	-- Show the information about what was done
	SELECT @old_conversation_handle AS old_handle, @new_conversation_handle AS new_handle, @message_body AS ResentRequest, @message_sequence_number AS sequence_number

	-- Get the nexet message in the sequence if one exists in dialog
	SELECT @message_sequence_number = MIN(message_sequence_number)
	FROM sys.transmission_queue
	WHERE to_service_name = '//MultiDB_Broker/TargetService'
	  AND from_service_name = N'//MultiDB_Broker/InitiatorService'
	  AND conversation_handle = @old_conversation_handle
	  AND message_sequence_number > @message_sequence_number;

	SET @message_body = NULL;

	-- Get the message body 
	SELECT @message_body = CAST(message_body AS XML)
	FROM sys.transmission_queue
	WHERE to_service_name = '//MultiDB_Broker/TargetService'
	  AND from_service_name = N'//MultiDB_Broker/InitiatorService'
	  AND conversation_handle = @old_conversation_handle
	  AND message_sequence_number = @message_sequence_number;


	END

END CONVERSATION @old_conversation_handle WITH CLEANUP;

COMMIT TRANSACTION;

SET @old_conversation_handle = NULL;

SELECT TOP 1 @old_conversation_handle = conversation_handle
FROM sys.transmission_queue
WHERE to_service_name = '//MultiDB_Broker/TargetService'
  AND from_service_name = N'//MultiDB_Broker/InitiatorService'

END
GO
