-- NOTE: THIS SCRIPT MUST BE RUN IN SQLCMD MODE TO FUNCTION PROPERLY
:CONNECT Buzz

-- Create a Service Broker endpoint
USE master;
GO
IF EXISTS (SELECT * FROM sys.endpoints
           WHERE name = N'SSB_Endpoint')
     DROP ENDPOINT SSB_Endpoint;
GO

CREATE ENDPOINT SSB_Endpoint
STATE = STARTED
AS TCP ( LISTENER_PORT = 4022 )
FOR SERVICE_BROKER (AUTHENTICATION = WINDOWS );
GO

-- Create the initiator database
USE master;
GO
IF DB_ID(N'InitiatorDB') IS NOT NULL
BEGIN
	ALTER DATABASE InitiatorDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE InitiatorDB;
END
GO
CREATE DATABASE InitiatorDB;
GO
USE InitiatorDB;
GO

-- Create the initiator master key
CREATE MASTER KEY ENCRYPTION BY PASSWORD = N'$tr0ngp@$$w0rd';
GO

-- Create the initiator user
CREATE USER InitiatorUser WITHOUT LOGIN;
GO

-- Create the initiator certificate
CREATE CERTIFICATE InitiatorCertificate
AUTHORIZATION InitiatorUser
WITH SUBJECT = N'Initiator Certificate',
	 EXPIRY_DATE = N'12/31/2020';

-- Delete the certificate backup if it exists
!! DEL "\\Buzz\Backups\InitiatorCertificate.cer"

-- Backup the initiator certificate
BACKUP CERTIFICATE InitiatorCertificate
TO FILE = N'\\Buzz\Backups\InitiatorCertificate.cer';
GO

-- Copy certificate backup to Target
!! COPY "\\Buzz\Backups\InitiatorCertificate.cer" \\Woody\Backups\

-- Create the message types
CREATE MESSAGE TYPE [//MultipleInstances/RequestMessage]
VALIDATION = WELL_FORMED_XML;

CREATE MESSAGE TYPE [//MultipleInstances/ReplyMessage]
VALIDATION = WELL_FORMED_XML;
GO

-- Create the contract
CREATE CONTRACT [//MultipleInstances/SimpleContract]
	([//MultipleInstances/RequestMessage] SENT BY INITIATOR,
	 [//MultipleInstances/ReplyMessage] SENT BY TARGET);
GO

-- Create the initiator queue and service
CREATE QUEUE InitiatorQueue;

CREATE SERVICE [//MultipleInstances/InitiatorService]
AUTHORIZATION InitiatorUser
ON QUEUE InitiatorQueue;
GO


-- Create references to target objects
CREATE USER TargetUser WITHOUT LOGIN;
GO

:CONNECT Buzz
USE InitiatorDB;
GO

-- Create the the Target Certificate from the backup file
CREATE CERTIFICATE TargetCertificate 
AUTHORIZATION TargetUser
FROM FILE = N'\\Buzz\Backups\TargetCertificate.cer'
GO

-- Create routes
-- This assumes that there are no duplicate service names in the target instance. 
-- If multiple databases on the target instance have services with the same name, 
-- use the BROKER_INSTANCE clause to specify the database on which you want to open
-- a conversation.

USE InitiatorDB;
GO

CREATE ROUTE TargetRoute
WITH SERVICE_NAME = N'//MultipleInstances/TargetService',
     ADDRESS = N'TCP://WOODY.SQLskillsDemos.com:4021';
GO

USE msdb;
GO

IF EXISTS (SELECT *
		   FROM sys.routes
		   WHERE name = N'InitiatorRoute')
	DROP ROUTE InitiatorRoute;
GO

CREATE ROUTE InitiatorRoute
WITH SERVICE_NAME = N'//MultipleInstances/InitiatorService',
     ADDRESS = N'LOCAL';
GO

USE InitiatorDB;
GO

CREATE REMOTE SERVICE BINDING TargetBinding
TO SERVICE N'//MultipleInstances/TargetService'
WITH USER = TargetUser;
GO