-- NOTE: THIS SCRIPT MUST BE RUN IN SQLCMD MODE TO FUNCTION PROPERLY
:CONNECT Woody

--Create a Service Broker endpoint
USE master;
GO

IF EXISTS (SELECT * 
		   FROM master.sys.endpoints
           WHERE name = N'SSB_Endpoint')
    DROP ENDPOINT SSB_Endpoint;
GO

CREATE ENDPOINT SSB_Endpoint
STATE = STARTED
AS TCP ( LISTENER_PORT = 4022 )
FOR SERVICE_BROKER (AUTHENTICATION = WINDOWS );
GO

-- Grant connect to the service account
GRANT CONNECT ON ENDPOINT::SSB_EndPoint to [SQLskillsDemos\SQLServiceAcct] ;
GO

GRANT CONNECT ON ENDPOINT::SSB_EndPoint TO [public];
GO

-- Create the target database
USE master;
GO
IF DB_ID(N'TargetDB') IS NOT NULL
BEGIN
	ALTER DATABASE TargetDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE TargetDB;
END
GO
CREATE DATABASE TargetDB;
GO
USE TargetDB;
GO

-- Create the master key
CREATE MASTER KEY ENCRYPTION BY PASSWORD = N'$tr0ngp@$$w0rd';
GO

-- Create the target user
CREATE USER TargetUser WITHOUT LOGIN;
GO

-- Create the target certificate
CREATE CERTIFICATE TargetCertificate 
AUTHORIZATION TargetUser
WITH SUBJECT = 'Target Certificate',
	 EXPIRY_DATE = N'12/31/2020';

-- Delete the certificate backup if it exists
!! DEL "\\Woody\Backups\TargetCertificate.cer"
GO
!! DEL "\\Woody\Backups\TargetCertificate.key"
GO

-- Backup the certificate public key to a file
BACKUP CERTIFICATE TargetCertificate
TO FILE = N'\\Woody\Backups\TargetCertificate.cer'
WITH PRIVATE KEY (
	FILE = N'\\Woody\Backups\TargetCertificate.key',
	ENCRYPTION BY PASSWORD = '997jkhUbhk$w4ez0876hKHJH5gh' );
GO

-- Copy certificate backup to Initiator
!! COPY \\Woody\Backups\TargetCertificate.cer \\Buzz\Backups\
!! COPY \\Woody\Backups\TargetCertificate.key \\Buzz\Backups\

-- Create the message types
CREATE MESSAGE TYPE [//MultipleInstances/RequestMessage]
VALIDATION = WELL_FORMED_XML;

CREATE MESSAGE TYPE [//MultipleInstances/ReplyMessage]
VALIDATION = WELL_FORMED_XML;
GO

-- Create the contract
CREATE CONTRACT [//MultipleInstances/SimpleContract]
	([//MultipleInstances/RequestMessage] SENT BY INITIATOR,
     [//MultipleInstances/ReplyMessage] SENT BY TARGET);
GO

-- Create the target queue and service
CREATE QUEUE TargetQueue;

CREATE SERVICE [//MultipleInstances/TargetService]
AUTHORIZATION TargetUser
ON QUEUE TargetQueue ([//MultipleInstances/SimpleContract]);
GO

ALTER AUTHORIZATION ON SERVICE::[//MultipleInstances/TargetService] TO TargetUser;
GO
